<?php
	class BitKassa_BitKassa_BitcoinController extends Mage_Core_Controller_Front_Action
	{
		protected $_bitkassa;
		protected $_model;

		// The possible bank return states
		const STATUS_PENDING	= 'open';
		const STATUS_PAID		= 'paid';		// paid (but not confirmed yet)
		const STATUS_CONFIRMED	= 'confirmed';	// paid and confirmed
		const STATUS_CANCELLED	= 'cancelled';	// user cancelled the payment
		const STATUS_FAILURE	= 'failure';	// an error occured
		const STATUS_EXPIRED	= 'expired';	// timeout

		public function _construct ()
		{		
			$this->_model = Mage::getModel('bitkassa/bitcoin');
			parent::_construct();
		}

		protected function _getCheckout() {
			return Mage::getSingleton('checkout/session');
		}

		protected function getBitcoinPaymentObject($merchant_key) {
			return new Bitcoin_Payment($merchant_key);
		}

		public function failureAction()
		{
		    $lastQuoteId = $this->_getCheckout()->getLastQuoteId();
		    $lastOrderId = $this->_getCheckout()->getLastOrderId();

		    if ($lastQuoteId && $lastOrderId) {
		        $orderModel = Mage::getModel('sales/order')->load($lastOrderId);		                   
	            $quote = Mage::getModel('sales/quote')->load($lastQuoteId);
	            $quote->setIsActive(true)->save();
	            $this->_redirect('checkout/index');
	            return;
		    }
		    if (!$lastQuoteId || !$lastOrderId) {
		        $this->_redirect('checkout/onepage/failure');
		        return;
		    }

		    $this->loadLayout();
		    $this->renderLayout();
		}

		public function paymentAction ()
		{		
			$orderIncrementId = $this->_getCheckout()->getLastRealOrderId();	
			$order = Mage::getModel('sales/order')->loadByIncrementId($orderIncrementId);
			$result = $this->generatePayUrl($order, $orderIncrementId);
			if(!$result['error']){
				$this->_redirectUrl($result['url']);
				return;
			}else{
				echo $result['error'];
			}
			exit;
		}

		public function statusAction ()
		{	
			$transaction_id = $this->getRequest()->getParam('transaction_id');
			if(!empty($transaction_id)){
				//Get payment object
				$bitkassa = $this->getBitcoinPaymentObject(Mage::helper('core')->decrypt(Mage::getStoreConfig('payment/bitkassa/merchant_key')));
				//Check status at gateway
				$bitkassa->checkPayment($transaction_id);
				//Update bitkassa_tx record
				Mage::getSingleton('bitkassa/bitcoin')->updatePayment($transaction_id, $bitkassa->getStatus());
				//Get payment object by tx id
				$payment = Mage::getSingleton('bitkassa/bitcoin')->getPaymentById($transaction_id);	
				if($payment['transaction_id']){
					if($payment['status'] == 'paid' || $payment['status'] == 'confirmed'){
						$order = Mage::getModel('sales/order')->loadByIncrementId($payment['order_id']);
						$order->sendNewOrderEmail();
						$this->_redirectUrl(Mage::getUrl('checkout/onepage/success'));
						return;
					}else{
						$this->_redirectUrl(Mage::getUrl('bitkassa/bitcoin/failure'));
						return;
					}
					exit;
				}else{
					echo 'Invalid transaction id';
				}				
			}
		}
		public function reportAction ()
		{		
			$transaction_id = $this->getRequest()->getParam('transaction_id');
			if(!empty($transaction_id)){
				$bitkassa = $this->getBitcoinPaymentObject(Mage::helper('core')->decrypt(Mage::getStoreConfig('payment/bitkassa/merchant_key')));		
				$bitkassa->checkPayment($transaction_id);
				$payment = Mage::getSingleton('bitkassa/bitcoin')->getPaymentById($transaction_id);	
				$order = Mage::getModel('sales/order')->loadByIncrementId($payment['order_id']);
				$correct_amount = ($bitkassa->getAmount() >= intval(round($order->getBaseGrandTotal()))-1);
				if($bitkassa->getStatus() != self::STATUS_PENDING && $bitkassa->getStatus() != self::STATUS_PAID){					
					switch ($bitkassa->getStatus()) {
						case self::STATUS_CONFIRMED:							
							if ($correct_amount){											
								$order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, true)->save();
							}else{
								$order->setState(Mage_Sales_Model_Order::STATE_CLOSED, true)->save(); // Fraud
							}
							break;
						case self::STATUS_CANCELLED:
							$order->setState(Mage_Sales_Model_Order::STATE_CANCELED, true)->save(); // Cancelled
							break;
						case self::STATUS_FAILURE:
							$order->setState(Mage_Sales_Model_Order::STATE_CANCELED, true)->save(); // Fail
							break;
						case self::STATUS_EXPIRED:
							$order->setState(Mage_Sales_Model_Order::STATE_CANCELED, true)->save(); // Expired
							break;
						default:
							$order->setState(Mage_Sales_Model_Order::STATE_CANCELED, true)->save(); // Fail
							break;
					}
				}
				Mage::getSingleton('bitkassa/bitcoin')->updatePayment($transaction_id, $bitkassa->getStatus());
			}
		}
		 public function generatePayUrl($order, $orderIncrementId)
		{          
			if(!$order->getBaseGrandTotal()) exit;
			$bitkassa    = $this->getBitcoinPaymentObject(Mage::helper('core')->decrypt(Mage::getStoreConfig('payment/bitkassa/merchant_key')));
			$amount      = intval(round($order->getBaseGrandTotal() * 100));
			$description = 'Order id: '.$orderIncrementId;
			$return_url  = Mage::getUrl('bitkassa/bitcoin/status');
			$report_url  = Mage::getUrl('bitkassa/bitcoin/report');
			$BitcoinPayment = $bitkassa->createPayment($amount, $description, $return_url, $report_url);
			try{
				if($BitcoinPayment){					
					$txid = $bitkassa->getTransactionId();
					Mage::getSingleton('bitkassa/bitcoin')->setPayment($order->increment_id, $txid);		
					return array('result' => 'success', 'url' => $bitkassa->getBitKassaURL(), 'error' => false);				
				}else{	
					throw new Exception($bitkassa->getErrorMessage());
				}
			} catch (Exception $e) {
				return array('result' => 'failed', 'url' => Mage::getUrl('checkout/cart'), 'error' => 'Kon geen Bitcoin betaling aanmaken, neem contact op met de beheerder.<br /><br/>
					Foutmelding voor de beheerder: ' . $e->getMessage()	
				
				);
			}	
		}
    }
?>